@@ -1,6 +1,6 @@
 """Low-level http related exceptions."""
 
-
+from textwrap import indent
 from typing import Optional, Union
 
 from .typedefs import _CIMultiDict
@@ -35,10 +35,11 @@ class HttpProcessingError(Exception):
         self.message = message
 
     def __str__(self) -> str:
-        return f"{self.code}, message={self.message!r}"
+        msg = indent(self.message, "  ")
+        return f"{self.code}, message:\n{msg}"
 
     def __repr__(self) -> str:
-        return f"<{self.__class__.__name__}: {self}>"
+        return f"<{self.__class__.__name__}: {self.code}, message={self.message!r}>"
 
 
 class BadHttpMessage(HttpProcessingError):
@@ -70,7 +71,11 @@ class TransferEncodingError(PayloadEncodingError):
 
 
 class ContentLengthError(PayloadEncodingError):
-    """Not enough data for satisfy content length header."""
+    """Not enough data to satisfy content length header."""
+
+
+class DecompressSizeError(PayloadEncodingError):
+    """Decompressed size exceeds the configured limit."""
 
 
 class LineTooLong(BadHttpMessage):
@@ -85,21 +90,27 @@ class LineTooLong(BadHttpMessage):
 
 class InvalidHeader(BadHttpMessage):
     def __init__(self, hdr: Union[bytes, str]) -> None:
-        if isinstance(hdr, bytes):
-            hdr = hdr.decode("utf-8", "surrogateescape")
-        super().__init__(f"Invalid HTTP Header: {hdr}")
-        self.hdr = hdr
+        hdr_s = hdr.decode(errors="backslashreplace") if isinstance(hdr, bytes) else hdr
+        super().__init__(f"Invalid HTTP header: {hdr!r}")
+        self.hdr = hdr_s
         self.args = (hdr,)
 
 
 class BadStatusLine(BadHttpMessage):
-    def __init__(self, line: str = "") -> None:
+    def __init__(self, line: str = "", error: Optional[str] = None) -> None:
         if not isinstance(line, str):
             line = repr(line)
-        super().__init__(f"Bad status line {line!r}")
+        super().__init__(error or f"Bad status line {line!r}")
         self.args = (line,)
         self.line = line
 
 
+class BadHttpMethod(BadStatusLine):
+    """Invalid HTTP method in status line."""
+
+    def __init__(self, line: str = "", error: Optional[str] = None) -> None:
+        super().__init__(line, error or f"Bad HTTP method in status line {line!r}")
+
+
 class InvalidURLError(BadHttpMessage):
     pass
